"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeAddCmd = exports.UnresolvedDependencyError = exports.EditorIncompatibleError = exports.InvalidPackumentDataError = void 0;
const package_url_1 = require("../domain/package-url");
const editor_version_1 = require("../domain/editor-version");
const package_reference_1 = require("../domain/package-reference");
const scoped_registry_1 = require("../domain/scoped-registry");
const project_manifest_1 = require("../domain/project-manifest");
const packument_version_resolving_1 = require("../packument-version-resolving");
const array_utils_1 = require("../utils/array-utils");
const ts_results_es_1 = require("ts-results-es");
const ts_custom_error_1 = require("ts-custom-error");
const error_logging_1 = require("./error-logging");
const dependency_logging_1 = require("./dependency-logging");
const registry_url_1 = require("../domain/registry-url");
const package_manifest_1 = require("../domain/package-manifest");
const packument_1 = require("../domain/packument");
class InvalidPackumentDataError extends ts_custom_error_1.CustomError {
    issue;
    _class = "InvalidPackumentDataError";
    constructor(issue) {
        super("A packument object was malformed.");
        this.issue = issue;
    }
}
exports.InvalidPackumentDataError = InvalidPackumentDataError;
class EditorIncompatibleError extends ts_custom_error_1.CustomError {
    _class = "EditorIncompatibleError";
    constructor() {
        super("A packuments target editor-version was not compatible with the installed editor-version.");
    }
}
exports.EditorIncompatibleError = EditorIncompatibleError;
class UnresolvedDependencyError extends ts_custom_error_1.CustomError {
    _class = "UnresolvedDependencyError";
    constructor() {
        super("A packuments dependency could not be resolved.");
    }
}
exports.UnresolvedDependencyError = UnresolvedDependencyError;
/**
 * Makes a {@link AddCmd} function.
 */
function makeAddCmd(parseEnv, resolveRemotePackumentVersion, resolveDependencies, loadProjectManifest, writeProjectManifest, log) {
    return async (pkgs, options) => {
        if (!Array.isArray(pkgs))
            pkgs = [pkgs];
        // parse env
        const envResult = await parseEnv(options);
        if (envResult.isErr()) {
            (0, error_logging_1.logEnvParseError)(log, envResult.error);
            return envResult;
        }
        const env = envResult.value;
        if (typeof env.editorVersion === "string")
            log.warn("editor.version", `${env.editorVersion} is unknown, the editor version check is disabled`);
        const tryAddToManifest = async function (manifest, pkg) {
            // is upstream package flag
            let isUpstreamPackage = false;
            // parse name
            const [name, requestedVersion] = (0, package_reference_1.splitPackageReference)(pkg);
            // packages that added to scope registry
            const pkgsInScope = Array.of();
            let versionToAdd = requestedVersion;
            if (requestedVersion === undefined || !(0, package_url_1.isPackageUrl)(requestedVersion)) {
                let resolveResult = await resolveRemotePackumentVersion(name, requestedVersion, env.registry).promise;
                if (resolveResult.isErr() && env.upstream) {
                    const upstreamResult = await resolveRemotePackumentVersion(name, requestedVersion, env.upstreamRegistry).promise;
                    if (upstreamResult.isOk()) {
                        resolveResult = upstreamResult;
                        isUpstreamPackage = true;
                    }
                    else {
                        resolveResult = (0, packument_version_resolving_1.pickMostFixable)(resolveResult, upstreamResult);
                    }
                }
                if (resolveResult.isErr()) {
                    (0, error_logging_1.logPackumentResolveError)(log, name, resolveResult.error);
                    return resolveResult;
                }
                const packumentVersion = resolveResult.value.packumentVersion;
                versionToAdd = packumentVersion.version;
                const targetEditorVersionResult = (0, package_manifest_1.tryGetTargetEditorVersionFor)(packumentVersion);
                if (targetEditorVersionResult.isErr()) {
                    log.warn("package.unity", `${targetEditorVersionResult.error.versionString} is not valid`);
                    if (!options.force) {
                        log.notice("suggest", "contact the package author to fix the issue, or run with option -f to ignore the warning");
                        return (0, ts_results_es_1.Err)(new InvalidPackumentDataError("Editor-version not valid."));
                    }
                }
                else {
                    const targetEditorVersion = targetEditorVersionResult.value;
                    // verify editor version
                    if (targetEditorVersion !== null &&
                        typeof env.editorVersion !== "string" &&
                        (0, editor_version_1.compareEditorVersion)(env.editorVersion, targetEditorVersion) < 0) {
                        log.warn("editor.version", `requires ${targetEditorVersion} but found ${(0, editor_version_1.stringifyEditorVersion)(env.editorVersion)}`);
                        if (!options.force) {
                            log.notice("suggest", `upgrade the editor to ${targetEditorVersion}, or run with option -f to ignore the warning`);
                            return (0, ts_results_es_1.Err)(new EditorIncompatibleError());
                        }
                    }
                }
                // pkgsInScope
                if (!isUpstreamPackage) {
                    log.verbose("dependency", `fetch: ${(0, package_reference_1.makePackageReference)(name, requestedVersion)}`);
                    const resolveResult = await resolveDependencies([env.registry, env.upstreamRegistry], name, requestedVersion, true);
                    if (resolveResult.isErr())
                        // TODO: Log errors
                        // TODO: Add tests
                        return resolveResult;
                    const [depsValid, depsInvalid] = resolveResult.value;
                    // add depsValid to pkgsInScope.
                    depsValid.forEach((dependency) => (0, dependency_logging_1.logValidDependency)(log, dependency));
                    depsValid
                        .filter((x) => {
                        const isUnityPackage = x.source === "built-in" || x.source === registry_url_1.unityRegistryUrl;
                        return !isUnityPackage;
                    })
                        .map((x) => x.name)
                        .forEach((name) => pkgsInScope.push(name));
                    // print suggestion for depsInvalid
                    let isAnyDependencyUnresolved = false;
                    depsInvalid.forEach((depObj) => {
                        (0, error_logging_1.logPackumentResolveError)(log, depObj.name, depObj.reason);
                        // If the manifest already has the dependency than it does not
                        // really matter that it was not resolved.
                        if (!(0, project_manifest_1.hasDependency)(manifest, depObj.name)) {
                            isAnyDependencyUnresolved = true;
                            if (depObj.reason instanceof packument_1.VersionNotFoundError)
                                log.notice("suggest", `to install ${(0, package_reference_1.makePackageReference)(depObj.name, depObj.reason.requestedVersion)} or a replaceable version manually`);
                        }
                    });
                    if (isAnyDependencyUnresolved) {
                        if (!options.force) {
                            log.error("missing dependencies", "please resolve the issue or run with option -f to ignore the warning");
                            return (0, ts_results_es_1.Err)(new UnresolvedDependencyError());
                        }
                    }
                }
                else
                    pkgsInScope.push(name);
            }
            // add to dependencies
            const oldVersion = manifest.dependencies[name];
            // Whether a change was made that requires overwriting the manifest
            let dirty = false;
            manifest = (0, project_manifest_1.setDependency)(manifest, name, versionToAdd);
            if (!oldVersion) {
                // Log the added package
                log.notice("manifest", `added ${(0, package_reference_1.makePackageReference)(name, versionToAdd)}`);
                dirty = true;
            }
            else if (oldVersion !== versionToAdd) {
                // Log the modified package version
                log.notice("manifest", `modified ${name} ${oldVersion} => ${versionToAdd}`);
                dirty = true;
            }
            else {
                // Log the existed package
                log.notice("manifest", `existed ${(0, package_reference_1.makePackageReference)(name, versionToAdd)}`);
            }
            if (!isUpstreamPackage && pkgsInScope.length > 0) {
                manifest = (0, project_manifest_1.mapScopedRegistry)(manifest, env.registry.url, (initial) => {
                    let updated = initial ?? (0, scoped_registry_1.makeEmptyScopedRegistryFor)(env.registry.url);
                    updated = pkgsInScope.reduce(scoped_registry_1.addScope, updated);
                    dirty =
                        !(0, array_utils_1.areArraysEqual)(updated.scopes, initial?.scopes ?? []) || dirty;
                    return updated;
                });
            }
            if (options.test)
                manifest = (0, project_manifest_1.addTestable)(manifest, name);
            return (0, ts_results_es_1.Ok)([manifest, dirty]);
        };
        // load manifest
        const loadResult = await loadProjectManifest(env.cwd).promise;
        if (loadResult.isErr()) {
            (0, error_logging_1.logManifestLoadError)(log, loadResult.error);
            return loadResult;
        }
        let manifest = loadResult.value;
        // add
        let dirty = false;
        for (const pkg of pkgs) {
            const result = await tryAddToManifest(manifest, pkg);
            if (result.isErr())
                return result;
            const [newManifest, manifestChanged] = result.value;
            if (manifestChanged) {
                manifest = newManifest;
                dirty = true;
            }
        }
        // Save manifest
        if (dirty) {
            const saveResult = await writeProjectManifest(env.cwd, manifest).promise;
            if (saveResult.isErr()) {
                (0, error_logging_1.logManifestSaveError)(log, saveResult.error);
                return saveResult;
            }
            // print manifest notice
            log.notice("", "please open Unity project to apply changes");
        }
        return (0, ts_results_es_1.Ok)(undefined);
    };
}
exports.makeAddCmd = makeAddCmd;
//# sourceMappingURL=cmd-add.js.map