"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeDepsCmd = void 0;
const package_url_1 = require("../domain/package-url");
const package_reference_1 = require("../domain/package-reference");
const common_errors_1 = require("../common-errors");
const ts_results_es_1 = require("ts-results-es");
const dependency_logging_1 = require("./dependency-logging");
const packument_1 = require("../domain/packument");
const error_logging_1 = require("./error-logging");
function errorPrefixForError(error) {
    if (error instanceof common_errors_1.PackumentNotFoundError)
        return "missing dependency";
    else if (error instanceof packument_1.VersionNotFoundError)
        return "missing dependency version";
    return "unknown";
}
/**
 * Makes a {@link DepsCmd} function.
 */
function makeDepsCmd(parseEnv, resolveDependencies, log) {
    return async (pkg, options) => {
        // parse env
        const envResult = await parseEnv(options);
        if (envResult.isErr()) {
            (0, error_logging_1.logEnvParseError)(log, envResult.error);
            return envResult;
        }
        const env = envResult.value;
        const [name, version] = (0, package_reference_1.splitPackageReference)(pkg);
        if (version !== undefined && (0, package_url_1.isPackageUrl)(version))
            // TODO: Convert to result
            throw new Error("Cannot get dependencies for url-version");
        const deep = options.deep || false;
        log.verbose("dependency", `fetch: ${(0, package_reference_1.makePackageReference)(name, version)}, deep=${deep}`);
        const resolveResult = await resolveDependencies([env.registry, env.upstreamRegistry], name, version, deep);
        if (resolveResult.isErr())
            // TODO: Log errors
            // TODO: Add tests
            return resolveResult;
        const [depsValid, depsInvalid] = resolveResult.value;
        depsValid.forEach((dependency) => (0, dependency_logging_1.logValidDependency)(log, dependency));
        depsValid
            .filter((x) => !x.self)
            .forEach((x) => log.notice("dependency", `${(0, package_reference_1.makePackageReference)(x.name, x.version)}`));
        depsInvalid
            .filter((x) => !x.self)
            .forEach((x) => {
            const prefix = errorPrefixForError(x.reason);
            log.warn(prefix, x.name);
        });
        return (0, ts_results_es_1.Ok)(undefined);
    };
}
exports.makeDepsCmd = makeDepsCmd;
//# sourceMappingURL=cmd-deps.js.map