"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeLoginCmd = void 0;
const npm_login_1 = require("../services/npm-login");
const registry_url_1 = require("../domain/registry-url");
const prompts_1 = require("./prompts");
const ts_results_es_1 = require("ts-results-es");
const error_logging_1 = require("./error-logging");
/**
 * Makes a {@link LoginCmd} function.
 */
function makeLoginCmd(parseEnv, getUpmConfigPath, login, log) {
    return async (options) => {
        // parse env
        const envResult = await parseEnv(options);
        if (envResult.isErr()) {
            (0, error_logging_1.logEnvParseError)(log, envResult.error);
            return envResult;
        }
        const env = envResult.value;
        // query parameters
        const username = options.username ?? (await (0, prompts_1.promptUsername)());
        const password = options.password ?? (await (0, prompts_1.promptPassword)());
        const email = options.email ?? (await (0, prompts_1.promptEmail)());
        const loginRegistry = options._global.registry !== undefined
            ? (0, registry_url_1.coerceRegistryUrl)(options._global.registry)
            : await (0, prompts_1.promptRegistryUrl)();
        const alwaysAuth = options.alwaysAuth || false;
        const configPathResult = await getUpmConfigPath(env.wsl, env.systemUser)
            .promise;
        if (configPathResult.isErr())
            return configPathResult;
        const configPath = configPathResult.value;
        const loginResult = await login(username, password, email, alwaysAuth, loginRegistry, configPath, options.basicAuth ? "basic" : "token", () => log.notice("auth", `you are authenticated as '${username}'`), (npmrcPath) => log.notice("config", `saved to npm config: ${npmrcPath}`)).promise;
        if (loginResult.isErr()) {
            const loginError = loginResult.error;
            if (loginError instanceof npm_login_1.AuthenticationError) {
                if (loginError.status === 401)
                    log.warn("401", "Incorrect username or password");
                else
                    log.error(loginError.status.toString(), loginError.message);
            }
            return loginResult;
        }
        log.notice("config", "saved unity config at " + configPath);
        return (0, ts_results_es_1.Ok)(undefined);
    };
}
exports.makeLoginCmd = makeLoginCmd;
//# sourceMappingURL=cmd-login.js.map