"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.logEnvParseError = exports.logPackumentResolveError = exports.logManifestSaveError = exports.logManifestLoadError = void 0;
const common_errors_1 = require("../common-errors");
const packument_1 = require("../domain/packument");
const file_io_1 = require("../io/file-io");
const wsl_1 = require("../io/wsl");
const process_1 = require("../utils/process");
const upm_config_io_1 = require("../io/upm-config-io");
/**
 * Logs a {@link ManifestLoadError} to the console.
 */
function logManifestLoadError(log, error) {
    const prefix = "manifest";
    if (error instanceof file_io_1.FsError && error.reason === file_io_1.FsErrorReason.Missing)
        log.error(prefix, `manifest at ${error.path} does not exist`);
    else if (error instanceof common_errors_1.FileParseError) {
        log.error(prefix, `failed to parse manifest at ${error.path}`);
        if (error.cause !== undefined)
            log.error(prefix, error.cause.message);
    }
}
exports.logManifestLoadError = logManifestLoadError;
/**
 * Logs a {@link ManifestWriteError} to the console.
 */
function logManifestSaveError(log, error) {
    const prefix = "manifest";
    log.error(prefix, "can not write manifest json file");
    log.error(prefix, error.message);
}
exports.logManifestSaveError = logManifestSaveError;
/**
 * Logs a {@link PackumentVersionResolveError} to the console.
 */
function logPackumentResolveError(log, packageName, error) {
    if (error instanceof common_errors_1.PackumentNotFoundError)
        log.error("404", `package not found: ${packageName}`);
    else if (error instanceof packument_1.VersionNotFoundError) {
        const versionList = [...error.availableVersions].reverse().join(", ");
        log.warn("404", `version ${error.requestedVersion} is not a valid choice of: ${versionList}`);
    }
}
exports.logPackumentResolveError = logPackumentResolveError;
/**
 * Logs a {@link EnvParseError} to a logger.
 */
function logEnvParseError(log, error) {
    // TODO: Formulate more specific error messages.
    const reason = error instanceof wsl_1.NoWslError
        ? "you attempted to use wsl even though you are not running openupm inside wsl"
        : error instanceof file_io_1.FsError
            ? error.reason === file_io_1.FsErrorReason.Missing
                ? `the file or directory at "${error.path}" does not exist`
                : `the file or directory at "${error.path}" could not be read`
            : error instanceof process_1.ChildProcessError
                ? "a required child process failed"
                : error instanceof common_errors_1.FileParseError
                    ? `the file at ${error.path} could not parsed into a ${error.targetDescription}`
                    : error instanceof upm_config_io_1.RequiredEnvMissingError
                        ? `none of the following environment variables were set: ${error.keyNames.join(", ")}`
                        : `a string was malformed. Expected to be ${error.formatName}`;
    const errorMessage = `environment information could not be parsed because ${reason}.`;
    log.error("", errorMessage);
    // TODO: Suggest actions user might take in order to fix the problem.
}
exports.logEnvParseError = logEnvParseError;
//# sourceMappingURL=error-logging.js.map