"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.stringifyEditorVersion = exports.makeEditorVersion = exports.tryParseEditorVersion = exports.compareEditorVersion = exports.isRelease = void 0;
function localeValue(loc) {
    if (loc === "c")
        return 1;
    throw new Error("Unknown locale");
}
function releaseValue(flag) {
    if (flag === "b")
        return 1;
    if (flag === "f")
        return 2;
    return 0;
}
function isPatch(version) {
    return "patch" in version;
}
function isRelease(version) {
    return isPatch(version) && "flag" in version;
}
exports.isRelease = isRelease;
function isLocal(version) {
    return isRelease(version) && "loc" in version;
}
/**
 * Compares two editor versions for ordering.
 * @param verA The first version.
 * @param verB The second version.
 * @returns A number indicating the ordering.
 */
const compareEditorVersion = function (verA, verB) {
    const editorVersionToArray = (ver) => [
        ver.major,
        ver.minor,
        isPatch(ver) ? ver.patch : 0,
        ...(isRelease(ver) ? [releaseValue(ver.flag), ver.build] : [0, 0]),
        ...(isLocal(ver)
            ? [ver.build, localeValue(ver.loc), ver.locBuild]
            : [0, 0, 0]),
    ];
    const arrA = editorVersionToArray(verA);
    const arrB = editorVersionToArray(verB);
    for (let i = 0; i < arrA.length; i++) {
        const valA = arrA[i];
        const valB = arrB[i];
        if (valA > valB)
            return 1;
        else if (valA < valB)
            return -1;
    }
    return 0;
};
exports.compareEditorVersion = compareEditorVersion;
/**
 * Attempts to parse editor version string to groups.
 */
const tryParseEditorVersion = function (version) {
    const regex = /^(?<major>\d+)\.(?<minor>[1234])(\.(?<patch>\d+)((?<flag>a|b|f|c)(?<build>\d+)((?<loc>c)(?<locBuild>\d+))?)?)?/;
    const match = regex.exec(version);
    if (!match)
        return null;
    const groups = match.groups;
    const regular = {
        major: parseInt(groups.major),
        minor: parseInt(groups.minor),
    };
    if (!groups.patch)
        return regular;
    const patch = {
        ...regular,
        patch: parseInt(groups.patch),
    };
    if (!(groups.flag && groups.build))
        return patch;
    const release = {
        ...patch,
        flag: groups.flag,
        build: parseInt(groups.build),
    };
    if (!(groups.loc && groups.locBuild))
        return release;
    return {
        ...release,
        loc: groups.loc,
        locBuild: parseInt(groups.locBuild),
    };
};
exports.tryParseEditorVersion = tryParseEditorVersion;
function makeEditorVersion(major, minor, patch, flag, build) {
    if (flag !== undefined)
        return { major, minor, patch, flag, build };
    if (patch !== undefined)
        return { major, minor, patch };
    return { major, minor };
}
exports.makeEditorVersion = makeEditorVersion;
/**
 * Converts an editor-version object to the corresponding string.
 * @param version The version to convert.
 */
function stringifyEditorVersion(version) {
    if (isLocal(version))
        return `${version.major}.${version.minor}.${version.patch}${version.flag}${version.build}${version.loc}${version.locBuild}`;
    else if (isRelease(version))
        return `${version.major}.${version.minor}.${version.patch}${version.flag}${version.build}`;
    else if (isPatch(version))
        return `${version.major}.${version.minor}.${version.patch}`;
    return `${version.major}.${version.minor}`;
}
exports.stringifyEditorVersion = stringifyEditorVersion;
//# sourceMappingURL=editor-version.js.map