"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryResolvePackumentVersion = exports.VersionNotFoundError = exports.NoVersionsError = exports.tryGetPackumentVersion = exports.tryGetLatestVersion = void 0;
const ts_custom_error_1 = require("ts-custom-error");
const ts_results_es_1 = require("ts-results-es");
const record_utils_1 = require("../utils/record-utils");
const hasLatestDistTag = (packument) => {
    return packument["dist-tags"]?.["latest"] !== undefined;
};
/**
 * Attempt to get the latest version from a package.
 * @param packument The package. All properties are assumed to be potentially
 * missing.
 */
const tryGetLatestVersion = function (packument) {
    if (hasLatestDistTag(packument))
        return packument["dist-tags"].latest;
    if (packument.version !== undefined)
        return packument.version;
    return undefined;
};
exports.tryGetLatestVersion = tryGetLatestVersion;
/**
 * Attempts to get a specific version from a packument.
 * @param packument The packument.
 * @param version The version to search.
 * @returns The packument-version or null if not found.
 */
function tryGetPackumentVersion(packument, version) {
    return packument.versions[version] ?? null;
}
exports.tryGetPackumentVersion = tryGetPackumentVersion;
/**
 * Error for when a packument with the searched name was found, but it
 * had no versions.
 */
class NoVersionsError extends ts_custom_error_1.CustomError {
    _class = "NoVersionsError";
    constructor() {
        super("A packument contained no versions");
    }
}
exports.NoVersionsError = NoVersionsError;
/**
 * Error for when a packument with the searched name was found, but a specific
 * requested version did not exist.
 */
class VersionNotFoundError extends ts_custom_error_1.CustomError {
    requestedVersion;
    availableVersions;
    _class = "VersionNotFoundError";
    constructor(
    /**
     * The version that was requested.
     */
    requestedVersion, 
    /**
     * A list of available versions.
     */
    availableVersions) {
        super("The requested version was not in the packument.");
        this.requestedVersion = requestedVersion;
        this.availableVersions = availableVersions;
    }
}
exports.VersionNotFoundError = VersionNotFoundError;
/**
 * Attempts to resolve a specific version from a packument. Resolve here means
 * that this function also accepts non-semantic versions and will attempt to
 * find the correct version from the packument. This differentiates it from
 * {@link tryGetPackumentVersion}.
 * @param packument The packument to search.
 * @param requestedVersion The requested version.
 */
function tryResolvePackumentVersion(packument, requestedVersion) {
    const availableVersions = (0, record_utils_1.recordKeys)(packument.versions);
    if (availableVersions.length === 0)
        return (0, ts_results_es_1.Err)(new NoVersionsError());
    // Find the latest version
    if (requestedVersion === undefined || requestedVersion === "latest") {
        let latestVersion = (0, exports.tryGetLatestVersion)(packument);
        if (latestVersion === undefined)
            latestVersion = availableVersions.at(-1);
        return (0, ts_results_es_1.Ok)(tryGetPackumentVersion(packument, latestVersion));
    }
    // Find a specific version
    if (!availableVersions.includes(requestedVersion))
        return (0, ts_results_es_1.Err)(new VersionNotFoundError(requestedVersion, availableVersions));
    return (0, ts_results_es_1.Ok)(tryGetPackumentVersion(packument, requestedVersion));
}
exports.tryResolvePackumentVersion = tryResolvePackumentVersion;
//# sourceMappingURL=packument.js.map