"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasDependency = exports.pruneManifest = exports.addTestable = exports.setScopedRegistry = exports.mapScopedRegistry = exports.tryGetScopedRegistryByUrl = exports.removeDependency = exports.setDependency = exports.emptyProjectManifest = void 0;
const string_utils_1 = require("../utils/string-utils");
const record_utils_1 = require("../utils/record-utils");
/**
 * Constructs an empty package-manifest.
 */
exports.emptyProjectManifest = { dependencies: {} };
/**
 * Sets a dependency in the manifest. If a dependency with that name already
 * exists, the version is overwritten.
 * @param manifest The manifest.
 * @param name The dependency name.
 * @param version The dependency version or url.
 */
function setDependency(manifest, name, version) {
    return {
        ...manifest,
        dependencies: { ...manifest.dependencies, [name]: version },
    };
}
exports.setDependency = setDependency;
/**
 * Removes a dependency from a manifest.
 * @param manifest The manifest.
 * @param name The dependency name.
 */
function removeDependency(manifest, name) {
    if (manifest.dependencies === undefined)
        return manifest;
    return {
        ...manifest,
        dependencies: (0, record_utils_1.removeRecordKey)(manifest.dependencies, name),
    };
}
exports.removeDependency = removeDependency;
/**
 * Attempts to get a scoped-registry with a specific url from the manifest.
 * @param manifest The manifest.
 * @param url The url.
 * @returns The scoped-registry or null if not found.
 */
function tryGetScopedRegistryByUrl(manifest, url) {
    function hasCorrectUrl(registry) {
        return (0, string_utils_1.removeTrailingSlash)(registry.url) === url;
    }
    return manifest.scopedRegistries?.find(hasCorrectUrl) ?? null;
}
exports.tryGetScopedRegistryByUrl = tryGetScopedRegistryByUrl;
/**
 * Updates a scoped-registry in a manifest by applying a mapping-function to it.
 * @param manifest The manifest.
 * @param registryUrl The url of the scoped-registry that should be mapped.
 * @param mapF The mapping function. Gets the scoped-registry with the given
 * registry-url as input, or null if the manifest has no scoped-registry with
 * that url. The scoped-registry returned by the function will be written back
 * to the manifest. If the function returns null the scoped-registry will be
 * removed from the manifest.
 * @returns The updated manifest.
 */
function mapScopedRegistry(manifest, registryUrl, mapF) {
    const updated = mapF(tryGetScopedRegistryByUrl(manifest, registryUrl));
    let newScopedRegistries = manifest.scopedRegistries?.filter((it) => it.url !== (updated?.url ?? registryUrl));
    if (updated !== null)
        newScopedRegistries = [...(newScopedRegistries ?? []), updated];
    return {
        ...manifest,
        scopedRegistries: newScopedRegistries,
    };
}
exports.mapScopedRegistry = mapScopedRegistry;
/**
 * Sets a scoped-registry in the manifest. If no scoped-registry with
 * the given url exists yet, it is added, otherwise it is overwritten.
 * @param manifest The manifest.
 * @param scopedRegistry The scoped-registry.
 */
function setScopedRegistry(manifest, scopedRegistry) {
    return mapScopedRegistry(manifest, scopedRegistry.url, () => scopedRegistry);
}
exports.setScopedRegistry = setScopedRegistry;
/**
 * Adds a testable to the manifest, if it is not already added.
 * @param manifest The manifest.
 * @param name The testable name.
 */
function addTestable(manifest, name) {
    if (manifest.testables?.includes(name))
        return manifest;
    return {
        ...manifest,
        testables: [...(manifest.testables ?? []), name].sort(),
    };
}
exports.addTestable = addTestable;
/**
 * Prunes the manifest by performing the following operations:
 *  - Remove scoped-registries without scopes.
 * @param manifest The manifest to prune.
 */
function pruneManifest(manifest) {
    return {
        ...manifest,
        scopedRegistries: manifest.scopedRegistries?.filter((it) => it.scopes.length > 0),
    };
}
exports.pruneManifest = pruneManifest;
/**
 * Checks if a manifest has a dependency on a specific package.
 */
function hasDependency(manifest, packageName) {
    return packageName in manifest.dependencies;
}
exports.hasDependency = hasDependency;
//# sourceMappingURL=project-manifest.js.map