"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryGetDirectoriesIn = exports.makeTextWriter = exports.makeTextReader = exports.FsError = exports.FsErrorReason = void 0;
const ts_results_es_1 = require("ts-results-es");
const promises_1 = __importDefault(require("fs/promises"));
const ts_custom_error_1 = require("ts-custom-error");
const error_type_guards_1 = require("../utils/error-type-guards");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
/**
 * Reason why a file-system operation failed.
 */
var FsErrorReason;
(function (FsErrorReason) {
    /**
     * Some generic reason.
     */
    FsErrorReason[FsErrorReason["Other"] = 0] = "Other";
    /**
     * The path did not exist.
     */
    FsErrorReason[FsErrorReason["Missing"] = 1] = "Missing";
})(FsErrorReason || (exports.FsErrorReason = FsErrorReason = {}));
/**
 * Generic error for when interacting with the file-system failed.
 */
class FsError extends ts_custom_error_1.CustomError {
    path;
    reason;
    // noinspection JSUnusedLocalSymbols
    _class = "FsError";
    constructor(
    /**
     * The path of the file or directory which caused the error.
     */
    path, 
    /**
     * The reason why the operation failed.
     */
    reason) {
        super("An interaction with the file-system caused an error.");
        this.path = path;
        this.reason = reason;
    }
}
exports.FsError = FsError;
function fsOperation(path, op) {
    return new ts_results_es_1.AsyncResult(ts_results_es_1.Result.wrapAsync(op)).mapErr((error) => {
        (0, error_type_guards_1.assertIsNodeError)(error);
        const cause = error.code === "ENOENT" ? FsErrorReason.Missing : FsErrorReason.Other;
        return new FsError(path, cause);
    });
}
/**
 * Makes a {@link ReadTextFile} function.
 */
function makeTextReader() {
    return (path) => fsOperation(path, () => promises_1.default.readFile(path, { encoding: "utf8" }));
}
exports.makeTextReader = makeTextReader;
/**
 * Makes a {@link WriteTextFile} function.
 */
function makeTextWriter() {
    return (filePath, content) => {
        const dirPath = path_1.default.dirname(filePath);
        return fsOperation(dirPath, () => fs_extra_1.default.ensureDir(dirPath)).andThen(() => fsOperation(filePath, () => promises_1.default.writeFile(filePath, content)));
    };
}
exports.makeTextWriter = makeTextWriter;
/**
 * Attempts to get the names of all directories in a directory.
 * @param directoryPath The directories name.
 */
function tryGetDirectoriesIn(directoryPath) {
    return fsOperation(directoryPath, () => promises_1.default.readdir(directoryPath, { withFileTypes: true }))
        .map((entries) => entries.filter((it) => it.isDirectory()))
        .map((directories) => directories.map((it) => it.name));
}
exports.tryGetDirectoriesIn = tryGetDirectoriesIn;
//# sourceMappingURL=file-io.js.map