"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.trySaveUpmConfig = exports.makeUpmConfigLoader = exports.makeUpmConfigPathGetter = exports.RequiredEnvMissingError = void 0;
const path_1 = __importDefault(require("path"));
const toml_1 = __importDefault(require("@iarna/toml"));
const ts_custom_error_1 = require("ts-custom-error");
const ts_results_es_1 = require("ts-results-es");
const file_io_1 = require("./file-io");
const env_util_1 = require("../utils/env-util");
const string_parsing_1 = require("../utils/string-parsing");
const wsl_1 = require("./wsl");
const configFileName = ".upmconfig.toml";
class RequiredEnvMissingError extends ts_custom_error_1.CustomError {
    keyNames;
    _class = "RequiredEnvMissingError";
    constructor(keyNames) {
        super(`Env was required to contain a value for one of the following keys, but all were missing: ${keyNames
            .map((keyName) => `"${keyName}"`)
            .join(", ")}.`);
        this.keyNames = keyNames;
    }
}
exports.RequiredEnvMissingError = RequiredEnvMissingError;
/**
 * Makes a {@link GetUpmConfigPath} function.
 */
function makeUpmConfigPathGetter(getHomePath) {
    function getConfigDirectory(wsl, systemUser) {
        const systemUserSubPath = "Unity/config/ServiceAccounts";
        if (wsl) {
            if (systemUser)
                return (0, wsl_1.tryGetWslPath)("ALLUSERSPROFILE").map((it) => path_1.default.join(it, systemUserSubPath));
            return (0, wsl_1.tryGetWslPath)("USERPROFILE");
        }
        if (systemUser) {
            const profilePath = (0, env_util_1.tryGetEnv)("ALLUSERSPROFILE");
            if (profilePath === null)
                return (0, ts_results_es_1.Err)(new RequiredEnvMissingError(["ALLUSERSPROFILE"])).toAsyncResult();
            return (0, ts_results_es_1.Ok)(path_1.default.join(profilePath, systemUserSubPath)).toAsyncResult();
        }
        return getHomePath().toAsyncResult();
    }
    return (wsl, systemUser) => {
        return getConfigDirectory(wsl, systemUser).map((directory) => path_1.default.join(directory, configFileName));
    };
}
exports.makeUpmConfigPathGetter = makeUpmConfigPathGetter;
/**
 * Makes a {@link LoadUpmConfig} function.
 */
function makeUpmConfigLoader(readFile) {
    return (configFilePath) => readFile(configFilePath)
        .andThen(string_parsing_1.tryParseToml)
        // TODO: Actually validate
        .map((toml) => toml)
        .orElse((error) => error instanceof file_io_1.FsError && error.reason === file_io_1.FsErrorReason.Missing
        ? (0, ts_results_es_1.Ok)(null)
        : (0, ts_results_es_1.Err)(error));
}
exports.makeUpmConfigLoader = makeUpmConfigLoader;
/**
 * Save the upm config.
 * @param config The config to save.
 * @param configFilePath The path of the file that should be saved to.
 */
const trySaveUpmConfig = (writeFile, config, configFilePath) => {
    const content = toml_1.default.stringify(config);
    return writeFile(configFilePath, content);
};
exports.trySaveUpmConfig = trySaveUpmConfig;
//# sourceMappingURL=upm-config-io.js.map