"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeResolveDependenciesService = void 0;
const domain_name_1 = require("../domain/domain-name");
const packument_cache_1 = require("../packument-cache");
const packument_version_resolving_1 = require("../packument-version-resolving");
const array_utils_1 = require("../utils/array-utils");
const package_manifest_1 = require("../domain/package-manifest");
const ts_results_es_1 = require("ts-results-es");
const common_errors_1 = require("../common-errors");
const errors_1 = require("npm-registry-fetch/lib/errors");
/**
 * Makes a {@link ResolveDependenciesService} function.
 */
function makeResolveDependenciesService(resolveRemovePackumentVersion, resolveLatestVersion) {
    // TODO: Add tests for this service
    return async (sources, name, version, deep) => {
        const latestVersionResult = version === undefined || version === "latest"
            ? await resolveLatestVersion(sources, name).promise
            : (0, ts_results_es_1.Ok)(version);
        if (latestVersionResult.isErr())
            return latestVersionResult;
        const latestVersion = latestVersionResult.value;
        // a list of pending dependency {name, version}
        const pendingList = Array.of([name, latestVersion]);
        // a list of processed dependency {name, version}
        const processedList = Array.of();
        // a list of dependency entry exists on the registry
        const depsValid = Array.of();
        // a list of dependency entry doesn't exist on the registry
        const depsInvalid = Array.of();
        // cached dict
        let packumentCache = packument_cache_1.emptyPackumentCache;
        async function tryResolveFromRegistry(registry, packumentName, version) {
            // First try cache
            const cacheResult = (0, packument_version_resolving_1.tryResolveFromCache)(packumentCache, registry.url, packumentName, version);
            if (cacheResult.isOk())
                return cacheResult;
            // Then registry
            return await resolveRemovePackumentVersion(packumentName, version, registry).promise;
        }
        while (pendingList.length > 0) {
            // NOTE: Guaranteed defined because of while loop logic
            const entry = pendingList.shift();
            const [entryName, entryVersion] = entry;
            const isProcessed = processedList.some((processed) => (0, array_utils_1.areArraysEqual)(processed, entry));
            if (!isProcessed) {
                // add entry to processed list
                processedList.push(entry);
                const isInternal = (0, domain_name_1.isInternalPackage)(entryName);
                const isSelf = entryName === name;
                if (isInternal) {
                    depsValid.push({
                        name: entryName,
                        version: latestVersion,
                        source: "built-in",
                        self: isSelf,
                    });
                    continue;
                }
                // Search all given registries.
                let resolveResult = (0, ts_results_es_1.Err)(new common_errors_1.PackumentNotFoundError());
                for (const source of sources) {
                    const result = await tryResolveFromRegistry(source, entryName, entryVersion);
                    if (result.isErr()) {
                        resolveResult = (0, packument_version_resolving_1.pickMostFixable)(resolveResult, result);
                        continue;
                    }
                    resolveResult = result;
                    break;
                }
                if (resolveResult.isErr()) {
                    if (resolveResult.error instanceof errors_1.HttpErrorBase)
                        return resolveResult;
                    depsInvalid.push({
                        name: entryName,
                        self: isSelf,
                        reason: resolveResult.error,
                    });
                    continue;
                }
                // Packument was resolved successfully
                const source = resolveResult.value.source;
                const resolvedPackumentVersion = resolveResult.value.packumentVersion;
                packumentCache = (0, packument_cache_1.addToCache)(packumentCache, resolveResult.value.source, resolveResult.value.packument);
                // add dependencies to pending list
                if (isSelf || deep) {
                    const dependencies = (0, package_manifest_1.dependenciesOf)(resolvedPackumentVersion);
                    pendingList.push(...dependencies);
                }
                const dependency = {
                    name: entryName,
                    version: entryVersion,
                    source,
                    self: isSelf,
                };
                depsValid.push(dependency);
            }
        }
        return (0, ts_results_es_1.Ok)([depsValid, depsInvalid]);
    };
}
exports.makeResolveDependenciesService = makeResolveDependenciesService;
//# sourceMappingURL=dependency-resolving.js.map