"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeParseEnvService = void 0;
const chalk_1 = __importDefault(require("chalk"));
const path_1 = __importDefault(require("path"));
const registry_url_1 = require("../domain/registry-url");
const upm_config_1 = require("../domain/upm-config");
const common_errors_1 = require("../common-errors");
const ts_results_es_1 = require("ts-results-es");
const env_util_1 = require("../utils/env-util");
const editor_version_1 = require("../domain/editor-version");
const file_io_1 = require("../io/file-io");
function determineCwd(getCwd, options) {
    return options._global.chdir !== undefined
        ? path_1.default.resolve(options._global.chdir)
        : getCwd();
}
function determineWsl(options) {
    return options._global.wsl === true;
}
function determinePrimaryRegistry(log, options, upmConfig) {
    const url = options._global.registry !== undefined
        ? (0, registry_url_1.coerceRegistryUrl)(options._global.registry)
        : options._global.cn === true
            ? (0, registry_url_1.makeRegistryUrl)("https://package.openupm.cn")
            : (0, registry_url_1.makeRegistryUrl)("https://package.openupm.com");
    if (upmConfig === null)
        return { url, auth: null };
    const auth = (0, upm_config_1.tryGetAuthForRegistry)(upmConfig, url);
    if (auth === null) {
        log.warn("env.auth", `failed to parse auth info for ${url} in .upmconfig.toml: missing token or _auth fields`);
    }
    return { url, auth };
}
function determineUpstreamRegistry(options) {
    const url = options._global.cn === true
        ? (0, registry_url_1.makeRegistryUrl)("https://packages.unity.cn")
        : (0, registry_url_1.makeRegistryUrl)("https://packages.unity.com");
    return { url, auth: null };
}
function determineLogLevel(options) {
    return options._global.verbose ? "verbose" : "notice";
}
function determineUseColor(options) {
    return options._global.color !== false && (0, env_util_1.tryGetEnv)("NODE_ENV") !== "test";
}
function determineUseUpstream(options) {
    return options._global.upstream !== false;
}
function determineIsSystemUser(options) {
    return options._global.systemUser === true;
}
/**
 * Creates a {@link ParseEnvService} function.
 */
function makeParseEnvService(log, getUpmConfigPath, loadUpmConfig, getCwd, loadProjectVersion) {
    return async (options) => {
        // log level
        log.level = determineLogLevel(options);
        // color
        const useColor = determineUseColor(options);
        if (!useColor) {
            chalk_1.default.level = 0;
            log.disableColor();
        }
        // upstream
        const upstream = determineUseUpstream(options);
        // region cn
        if (options._global.cn === true)
            log.notice("region", "cn");
        // auth
        const systemUser = determineIsSystemUser(options);
        const wsl = determineWsl(options);
        // registries
        const upmConfigResult = await getUpmConfigPath(wsl, systemUser).andThen(loadUpmConfig).promise;
        if (upmConfigResult.isErr())
            return upmConfigResult;
        const upmConfig = upmConfigResult.value;
        const registry = determinePrimaryRegistry(log, options, upmConfig);
        const upstreamRegistry = determineUpstreamRegistry(options);
        // cwd
        const cwd = determineCwd(getCwd, options);
        // editor version
        const projectVersionLoadResult = await loadProjectVersion(cwd).promise;
        if (projectVersionLoadResult.isErr()) {
            const error = projectVersionLoadResult.error;
            if (error instanceof file_io_1.FsError && error.reason === file_io_1.FsErrorReason.Missing)
                log.warn("ProjectVersion", `can not locate ProjectVersion.text at path ${error.path}`);
            else if (error instanceof common_errors_1.FileParseError)
                log.error("ProjectVersion", "ProjectVersion.txt could not be parsed for editor-version!");
            return projectVersionLoadResult;
        }
        const unparsedEditorVersion = projectVersionLoadResult.value;
        const parsedEditorVersion = (0, editor_version_1.tryParseEditorVersion)(unparsedEditorVersion);
        const editorVersion = parsedEditorVersion !== null && (0, editor_version_1.isRelease)(parsedEditorVersion)
            ? parsedEditorVersion
            : unparsedEditorVersion;
        return (0, ts_results_es_1.Ok)({
            cwd,
            editorVersion,
            registry,
            systemUser,
            upstream,
            upstreamRegistry,
            wsl,
        });
    };
}
exports.makeParseEnvService = makeParseEnvService;
//# sourceMappingURL=parse-env.js.map